function partitionStruct = createPartition(nSamples, testPct, testNum, cvNum)
%createPartition - create test/train/cross-validation partition structure.
% Create random test/train/cross-validation partitions in the format of the
% Similarity-based Learning Repository.
%
% USAGE
% partitionStruct = createPartition(nSamples, testPct, testNum, cvNum)
% 
% nSamples: total number of data samples.
% testPct: percent of sample to reserve as test samples (e.g. 20 for 20%).
% testNum: number of random partitions to create.
% cvNum: number of cross-validation partitions for each test partition.
% partitionStruct: structure suitable for writePartitionFile(), with fields
%     testNum: the testNum input
%     cvNum: the cvNum input
%     testIdx: testNum-by-1 cell array, each element containing indices of
%       the test samples.
%     cvIdx: testNum-by-cvNum cell array, each element containing indices
%     of training samples to be reserved for testing in a cross-validation
%     iteration.
%
% EXAMPLE
% Partition 100 samples into 20 random train/test splits, reserving 20% of
% the saples for testing, and doing 10 cross-validation iterations from the
% training set.
% partitionStruct = createPartition(100, 20, 20, 10);
%
% See also writePartitionFile().

% Written by Luca Cazzanti, Engineer, Applied Physics Lab, March 2010.


nTestSamples = ceil(testPct/100 * nSamples);
testIdx = cell(testNum,1);
cvIdx = cell(testNum, cvNum);

for iTest = 1:testNum
  randIdx = randperm(nSamples) - 1; % using C-style array indexing
  testIdx{iTest} = randIdx(1:nTestSamples);
  trainIdx = randIdx( (nTestSamples+1):end );
  nTrain = length(trainIdx);
  
  numPerCv = ceil(nTrain/cvNum);
  cvBeginIdx = 1;
  for iCrossVal = 1:(cvNum-1)
    cvIdx{iTest, iCrossVal} = trainIdx(cvBeginIdx:(cvBeginIdx + numPerCv - 1));
    cvBeginIdx = cvBeginIdx + numPerCv;
  end
  cvIdx{iTest, cvNum} = trainIdx(cvBeginIdx:end);
  
end

% The last partition


partitionStruct.testNum = testNum;
partitionStruct.cvNum = cvNum;
partitionStruct.testIdx = testIdx;
partitionStruct.cvIdx = cvIdx;

return