function [bestSize, bestSigma, bestEta, minErr] = ...
  crossValidateParamsClassMtmrPairwiseLocalSda( neighborhoodSizeVec, sigmaVec, ...
  etaVec, simMat, crossValIdxMat, classLabels, simDomain, testIdx)
%crossValidateParamsClassMtmrPairwiseLocalSda
% TO DO - COPIED FROM ELSEWHERE
% Cross-validate the neighborhood size for the local SDA classifier, using
% the N-fold cross validation strategy. This function is designed to be
% used in conjucntion with readPartitionFile() and with the partition files
% from the Similarity Learning Repository.
%
% Assumes similarity data are used (not dissimilarity) and the centroid
% type is 'maxSumSim'.
%
% USAGE
% [bestSize, minErr, sameClassBestSize, saneClassMinErr] = ...
%     crossvalidateNeighborhoodLocalSda( ...
%     neighborhoodSizeVec, simMat, crossValIdxMat, classLabels, ...
%     simDomain, testIdx)
% neighbrohoodSizeVec: vector of neighborhood sizes to cross-validate.
% simMat: N-by-N matrix of pairwise similarities.
% crossValIdxMat: N-by-K matrix of indices into the rows of simMat,
%    denoting the samples to be used for cross-validation. It is a cell
%    from crossValIdxCell as returned by readPartitionFile().
% classLabels: N-by-1 vector of class labels for each row of simMat.
% simDomain: vector of possible similarity values.
% testIdx: a vector if indices into the columns of simMat denoting which
%     columns should be reserved for the testing data set.
% bestSize: optimized neighborhood size.
% minErr: cross-validation error corresponding to best neighborhood size.
% sameClassBestSize: optimized neighborhood size for same-class SDA.
% sameClassMinErr: cross-validation error corresponding to best 
%     neighborhood size for same-class SDA.

% Written by Luca Cazzanti, engineer.
% Applied Physics Laboratory, University of Washington, Feb 2007.
%$Id:$
%$DateTime:$
%$Author:$

if isempty(simDomain)
  simDomain = unique(simMat(:));
else
  simDomain = unique(simDomain);
end

% Put params in vectors
neighborhoodSizeVec = sort(neighborhoodSizeVec);
nNeighborhoodSizes = length(neighborhoodSizeVec);

sigmaVec = sort(sigmaVec);
nSigmas = length(sigmaVec);

etaVec = sort(etaVec);
nEtas = length(etaVec);

[nData, notUsed] = size(simMat); %#ok
[nFolds, notUsed_nCrossVals] = size(crossValIdxMat); %#ok

neighborIdx = findNearNeighbors(simMat, size(simMat,2), 'similarity');
neighborIdx = adjustSelfSimilarityRank(neighborIdx);
neighborIdx(:,1) = [];

errMat = -10e10 * ones(nNeighborhoodSizes, nSigmas, nEtas); 

    
for iSize = 1:nNeighborhoodSizes
  
  thisSize = neighborhoodSizeVec(iSize);
  disp(['Size: ' num2str(thisSize) ]);
  thisError = 0;
  for iFold = 1:nFolds
    crossValIdx = crossValIdxMat(iFold,:);
    crossValIdx(crossValIdx == 0) = []; % take out any remaining filler values
    nCrossVals = length(crossValIdx);
    
    thisFoldClassEst = -1e6 * ones(nEtas, nSigmas, nCrossVals);
    thisFoldError = -1e6 * ones(nEtas, nSigmas, nCrossVals);
        
    % iterate through all the "test" samples in this cross-validation fold
    for iCrossVal = 1:nCrossVals
      
      % get the ith "test" sample's index
      thisCrossValSample = crossValIdx(iCrossVal);
      % pull out its neighbors
      neighborsToThisCrossValSample = neighborIdx(thisCrossValSample,:);
      
      % DEBUG CODE, DELETE FOR RUNNING
      %save neighborsToThisCrossValSample
      %save testIdx
      %save crossValIdx
      
      % Remove other cross validation "test" points as candidate neighbors
      % by looking through each of them and removing them
      for iCount = 1:nCrossVals
        %idx = find( neighborsToThisCrossValSample == crossValIdx(iCount)
        %);
        idx =  neighborsToThisCrossValSample == crossValIdx(iCount);
        neighborsToThisCrossValSample(idx) = [];
      end % for iCount
      
      % Begin Maya's code fix
      % Remove the true test samples as candidate neighbors
      % by looking through each of them and removing them
      
      for iTest = 1:length(testIdx)
        %idx = find( neighborsToThisCrossValSample == testIdx(iTest) );
        idx = neighborsToThisCrossValSample == testIdx(iTest) ;
        neighborsToThisCrossValSample(idx) = [];
      end % for iTest
      
      % end Maya's code fix
      
      neighborsToPick = neighborsToThisCrossValSample(1:thisSize);
      neighborClassLabels = classLabels(neighborsToPick);
      testPointSim = simMat( thisCrossValSample, neighborsToPick );
      neighborSimMat = simMat( :, neighborsToPick );
      neighborSimMat = neighborSimMat( neighborsToPick, : );
      
      [thisFoldClassEst(:, :, iCrossVal)] = ...
        classifyClassMtmrPairwiseLocalSda(testPointSim, neighborSimMat, ...
        neighborClassLabels, sigmaVec, etaVec, simDomain);
      thisFoldError(:, :, iCrossVal) = ...
        thisFoldClassEst(:, :, iCrossVal) ~= classLabels(thisCrossValSample);
      
    end %for iCrossVal
    
    thisError = thisError + sum(thisFoldError, 3);
    
  end % for iFold
  errMat(iSize, :, :) = thisError.';
  
end %for iSize
% end % for iEta
%end % for iSigma


% Get parameter values corresponding to minimum error
[minErrTemp1, minErrEtaIdx] = min(errMat,[], 3);
[minErrTemp2, minErrSigmaIdx] = min(minErrTemp1, [],2);
[minErr, minErrSizeIdx] = min(minErrTemp2);

bestEta = etaVec( minErrEtaIdx( minErrSizeIdx, minErrSigmaIdx(minErrSizeIdx) ) );
bestSigma = sigmaVec(minErrSigmaIdx(minErrSizeIdx));
bestSize = neighborhoodSizeVec(minErrSizeIdx);


return