// File: RealMatrix.cpp
// Purpose: Implementation of the RealMatrix class
// Author: Yihua Chen (yhchen@u.washington.edu)
// Last modified: 5/24/2008 7:37:44 PM

#include <cmath>
#include <algorithm>
#include <map>
#include "RealMatrix.h"

using namespace std;

namespace idl
{
	// Get the sum of all the elements
	double RealMatrix::sum() const
	{
		double s = 0.0;

		for (vector<double>::const_iterator itr = elements.begin();
			 itr != elements.end(); ++itr)
		{
			s += *itr;
		}

		return s;
	}

	// Get the mean of all the elements
	double RealMatrix::mean() const
	{
		if (0 == nRows || 0 == nColumns)
		{
			return 0.0;
		}

		return (sum() / static_cast<double>(elements.size()));
	}

	// Get the variance of all the elements
	double RealMatrix::var() const
	{
		double m = mean();
		double v = 0.0;

		for (vector<double>::const_iterator itr = elements.begin();
			 itr != elements.end(); ++itr)
		{
			double d = *itr - m;
			v += d * d;
		}

		v /= static_cast<double>(elements.size());

		return (v > 0.0) ? v : 0.0; // To counter numerical instability
	}

	// Get the standard deviation of all the elements
	double RealMatrix::std() const
	{
		return sqrt(var());
	}

	// Get the maximum of all the elements
	double RealMatrix::max() const
	{
		vector<double>::const_iterator itr =
			max_element(elements.begin(), elements.end());

		return *itr;
	}

	// Get the minimum of all the elements
	double RealMatrix::min() const
	{
		vector<double>::const_iterator itr =
			min_element(elements.begin(), elements.end());

		return *itr;
	}

	// Get the sum of a row
	double RealMatrix::sumRow(unsigned int i) const
	{
		double s = 0.0;

		unsigned int a = i * nColumns;
		unsigned int b = a + nColumns; // (i + 1) * nColumns

		for (unsigned int k = a; k < b; ++k)
		{
			s += elements[k];
		}

		return s;
	}

	// Get the mean of a row
	double RealMatrix::meanRow(unsigned int i) const
	{
		if (0 == nColumns)
		{
			return 0.0;
		}

		return sumRow(i) / static_cast<double>(nColumns);
	}

	// Get the maximum of a row
	double RealMatrix::maxRow(unsigned int i) const
	{
		unsigned int a = i * nColumns;
		unsigned int b = a + nColumns; // (i + 1) * nColumns

		double v = elements[a];

		for (unsigned int k = a + 1; k < b; ++k)
		{
			if (elements[k] > v)
			{
				v = elements[k];
			}
		}

		return v;
	}

	// Get the minimum of a row
	double RealMatrix::minRow(unsigned int i) const
	{
		unsigned int a = i * nColumns;
		unsigned int b = a + nColumns; // (i + 1) * nColumns

		double v = elements[a];

		for (unsigned int k = a + 1; k < b; ++k)
		{
			if (elements[k] < v)
			{
				v = elements[k];
			}
		}

		return v;
	}

	// Get the sum of a column
	double RealMatrix::sumColumn(unsigned int j) const
	{
		double s = 0.0;

		unsigned int b = nRows * nColumns;

		for (unsigned int k = j; k < b; k += nColumns)
		{
			s += elements[k];
		}

		return s;
	}

	// Get the mean of a column
	double RealMatrix::meanColumn(unsigned int j) const
	{
		if (0 == nRows)
		{
			return 0.0;
		}

		return sumColumn(j) / static_cast<double>(nRows);
	}

	// Get the maximum of a column
	double RealMatrix::maxColumn(unsigned int j) const
	{
		unsigned int b = nRows * nColumns;

		double v = elements[j];

		for (unsigned int k = j + nColumns; k < b; k += nColumns)
		{
			if (elements[k] > v)
			{
				v = elements[k];
			}
		}

		return v;
	}

	// Get the minimum of a column
	double RealMatrix::minColumn(unsigned int j) const
	{
		unsigned int b = nRows * nColumns;

		double v = elements[j];

		for (unsigned int k = j + nColumns; k < b; k += nColumns)
		{
			if (elements[k] < v)
			{
				v = elements[k];
			}
		}

		return v;
	}

	// Get all unique values
	// Returns a Mx1 RealMatrix of unique values
	void RealMatrix::unique(RealMatrix &uniqueVals) const//vector<double> *uniqueVals)
	{

		map<double,int> seen;
		for (vector<double>::const_iterator itr = elements.begin();
			itr != elements.end(); ++itr)
			seen[(*itr)] = 1;
		vector<double> uniqueValsVect(seen.size());
		
		unsigned int i = 0;
		for (map<double,int>::const_iterator itr = seen.begin();
			itr != seen.end(); ++itr)
		{
			uniqueValsVect[i] = itr->first;
			++i;
		}
		
		uniqueVals.resize((unsigned int)uniqueValsVect.size(),1);
		i = 0;
		for(vector<double>::const_iterator itr = uniqueValsVect.begin();
			itr != uniqueValsVect.end(); ++itr)
		{
			uniqueVals(i++,0) = *itr;
		}
	}

	// Compute the function on each element
	void RealMatrix::fxn(double (*func)(double)) 
	{
		for (vector<double>::iterator itr = elements.begin();
			 itr != elements.end(); ++itr)
		{
			*itr = (*func)(*itr);
		}
		//return *this;
	}

	// Create a matrix of all ones
	RealMatrix & RealMatrix::ones()
	{
		fill(1.0);

		return *this;
	}

	// Create a matrix of all ones
	RealMatrix & RealMatrix::ones(unsigned int m, unsigned int n)
	{
		nRows = m;
		nColumns = n;

		elements.clear();
		elements.resize(m * n, 1.0);

		return *this;
	}

	// Create a matrix of all ones
	RealMatrix & RealMatrix::ones(unsigned int m)
	{
		return ones(m, m);
	}

	// Create a matrix of all zeros
	RealMatrix & RealMatrix::zeros()
	{
		fill(0.0);

		return *this;
	}

	// Create a matrix of all zeros
	RealMatrix & RealMatrix::zeros(unsigned int m, unsigned int n)
	{
		nRows = m;
		nColumns = n;

		elements.clear();
		elements.resize(m * n, 0.0);

		return *this;
	}

	// Create a matrix of all zeros
	RealMatrix & RealMatrix::zeros(unsigned int m)
	{
		return zeros(m, m);
	}

	// Create an identity matrix
	RealMatrix & RealMatrix::eye()
	{
		zeros();

		unsigned int p = (nRows < nColumns) ? nRows : nColumns;

		for (unsigned int i = 0; i < p; ++i)
		{
			elements[i * (nColumns + 1)] = 1.0;
		}

		return *this;
	}

	// Create an identity matrix
	RealMatrix & RealMatrix::eye(unsigned int m)
	{
		zeros(m);

		for (unsigned int i = 0; i < m; ++i)
		{
			elements[i * (m + 1)] = 1.0;
		}

		return *this;
	}

	// Create an identity matrix
	RealMatrix & RealMatrix::eye(unsigned int m, unsigned int n)
	{
		zeros(m, n);

		unsigned int p = (m < n) ? m : n;

		for (unsigned int i = 0; i < p; ++i)
		{
			elements[i * (n + 1)] = 1.0;
		}

		return *this;
	}

	// Negate all the elements
	RealMatrix & RealMatrix::negate()
	{
		for (vector<double>::iterator itr = elements.begin();
			 itr != elements.end(); ++itr)
		{
			*itr = -(*itr);
		}

		return *this;
	}

	// Unary negation
	RealMatrix RealMatrix::operator - () const
	{
		RealMatrix mat(*this);

		mat.negate();

		return mat;
	}

	RealMatrix & RealMatrix::operator += (double v)
	{
		for (vector<double>::iterator itr = elements.begin();
			 itr != elements.end(); ++itr)
		{
			*itr += v;
		}

		return *this;
	}

	RealMatrix & RealMatrix::operator -= (double v)
	{
		for (vector<double>::iterator itr = elements.begin();
			 itr != elements.end(); ++itr)
		{
			*itr -= v;
		}

		return *this;
	}

	RealMatrix & RealMatrix::operator *= (double v)
	{
		for (vector<double>::iterator itr = elements.begin();
			 itr != elements.end(); ++itr)
		{
			*itr *= v;
		}

		return *this;
	}

	RealMatrix & RealMatrix::operator /= (double v)
	{
		for (vector<double>::iterator itr = elements.begin();
			 itr != elements.end(); ++itr)
		{
			*itr /= v;
		}

		return *this;
	}

	RealMatrix & RealMatrix::operator += (const RealMatrix &A)
	{
		if (nRows == A.nRows && nColumns == A.nColumns)
		{
			vector<double>::iterator itr;
			vector<double>::const_iterator itrA;
			for (itr = elements.begin(), itrA = A.elements.begin();
				 itr != elements.end() && itrA != A.elements.end();
				 ++itr, ++itrA)
			{
				*itr += *itrA;
			}
		}

		return *this;
	}

	RealMatrix & RealMatrix::operator -= (const RealMatrix &A)
	{
		if (nRows == A.nRows && nColumns == A.nColumns)
		{
			vector<double>::iterator itr;
			vector<double>::const_iterator itrA;
			for (itr = elements.begin(), itrA = A.elements.begin();
				 itr != elements.end() && itrA != A.elements.end();
				 ++itr, ++itrA)
			{
				*itr -= *itrA;
			}
		}

		return *this;
	}

	RealMatrix & RealMatrix::operator *= (const RealMatrix &A)
	{
		RealMatrix B = (*this) * A;

		nRows = B.nRows;
		nColumns = B.nColumns;
		elements = B.elements;

		return *this;
	}

	// Get the trace
	double RealMatrix::trace() const
	{
		if (!isSquare())
		{
			return 0.0;
		}

		double tr = 0.0;

		for (unsigned int i = 0; i < nRows; ++i)
		{
			tr += elements[i * nColumns + i];
		}

		return tr;
	}

	RealMatrix operator + (const RealMatrix &A, double v)
	{
		RealMatrix B(A);
		B += v;

		return B;
	}

	RealMatrix operator - (const RealMatrix &A, double v)
	{
		RealMatrix B(A);
		B -= v;

		return B;
	}

	RealMatrix operator * (const RealMatrix &A, double v)
	{
		RealMatrix B(A);
		B *= v;

		return B;
	}

	RealMatrix operator / (const RealMatrix &A, double v)
	{
		RealMatrix B(A);
		B /= v;

		return B;
	}

	RealMatrix operator + (double v, const RealMatrix &A)
	{
		RealMatrix B(A);
		B += v;

		return B;
	}

	RealMatrix operator - (double v, const RealMatrix &A)
	{
		RealMatrix B(A);
		B -= v;
		B.negate();

		return B;
	}

	RealMatrix operator * (double v, const RealMatrix &A)
	{
		RealMatrix B(A);
		B *= v;

		return B;
	}

	RealMatrix operator + (const RealMatrix &A, const RealMatrix &B)
	{
		RealMatrix C(A);
		C += B;

		return C;
	}

	RealMatrix operator - (const RealMatrix &A, const RealMatrix &B)
	{
		RealMatrix C(A);
		C -= B;

		return C;
	}

	RealMatrix operator * (const RealMatrix &A, const RealMatrix &B)
	{
		RealMatrix C;

		if (A.getColumnNum() == B.getRowNum())
		{
			unsigned int m = A.getRowNum();
			unsigned int n = B.getColumnNum();
			unsigned int p = A.getColumnNum();

			C.zeros(m, n);
			
			for (unsigned int i = 0; i < m; ++i)
			{
				for (unsigned int j = 0; j < n; ++j)
				{
					for (unsigned int k = 0; k < p; ++k)
					{
						C(i, j) += A(i, k) * B(k, j);
					}
				}
			}
		}

		return C;
	}

	RealMatrix RealMatrix::dot(const RealMatrix &B)
	{
		RealMatrix C;

		RealMatrix A = *this;
		if (A.getColumnNum() == B.getColumnNum() && A.getRowNum() == B.getRowNum())
		{
			unsigned int m = A.getRowNum();
			unsigned int n = B.getColumnNum();

			C.zeros(m, n);
			
			for (unsigned int i = 0; i < m; ++i)
			{
				for (unsigned int j = 0; j < n; ++j)
				{
					C(i, j) += A(i, j) * B(i, j);
					
				}
			}
		}

		return C;
	}

	CvMat *RealMatrix::dumpCvMat() const
	{
		CvMat *mat = cvCreateMat(nRows, nColumns, CV_64FC1);

		vector<double>::const_iterator itr = elements.begin();

		for (unsigned int i = 0; i < nRows; ++i)
		{
			for (unsigned int j = 0; j < nColumns; ++j)
			{
				cvmSet(mat, i, j, *itr++);
			}
		}

		return mat;
	}

	void RealMatrix::swallowCvMat(const CvMat *mat)
	{
		resize(mat->rows, mat->cols);

		vector<double>::iterator itr = elements.begin();

		for (int i = 0; i < mat->rows; ++i)
		{
			for (int j = 0; j < mat->cols; ++j)
			{
				*itr++ = cvmGet(mat, i, j);
			}
		}
	}


        void RealMatrix::swallowFlatArray(const double *mat, unsigned int nElem)
	{
	  resize(1, nElem);

	  vector<double>::iterator itr = elements.begin();
	  
	  for (int i = 0; i < nElem; ++i)
	    {
	      *itr++ = *mat++;
	    }
	}



	// Solve a linear system of equations: Ax = b
	bool solve(const RealMatrix &A, const RealMatrix &b, RealMatrix &x, bool svd)
	{
		if(A.getRowNum() != b.getRowNum()) return false;

		CvMat *_A = A.dumpCvMat();
		CvMat *_b = b.dumpCvMat();
		CvMat *_x = cvCreateMat(A.getColumnNum(), b.getColumnNum(), CV_64FC1);

		if (svd) // SVD method
		{
			if (0 == cvSolve(_A, _b, _x, CV_SVD)) return false;
		}
		else // LU decomposition method
		{
			if (0 == cvSolve(_A, _b, _x)) return false;
		}

		x.swallowCvMat(_x);

		cvReleaseMat(&_A);
		cvReleaseMat(&_b);
		cvReleaseMat(&_x);

		return true;
	}

	void RealMatrix::eig(RealMatrix &d) const
	{
		CvMat *_A = dumpCvMat();
		CvMat *_X = cvCreateMat(nRows, nColumns, CV_64FC1);
		CvMat *_d = cvCreateMat(nRows, 1, CV_64FC1);

		cvEigenVV(_A, _X, _d);

		d.swallowCvMat(_d);

		cvReleaseMat(&_A);
		cvReleaseMat(&_X);
		cvReleaseMat(&_d);
	}

	void RealMatrix::eig(RealMatrix &V, RealMatrix &d) const
	{
		CvMat *_A = dumpCvMat();
		CvMat *_X = cvCreateMat(nRows, nColumns, CV_64FC1);
		CvMat *_d = cvCreateMat(nRows, 1, CV_64FC1);

		cvEigenVV(_A, _X, _d);

		V.swallowCvMat(_X);
		d.swallowCvMat(_d);

		cvReleaseMat(&_A);
		cvReleaseMat(&_X);
		cvReleaseMat(&_d);
	}
}
