function neighborIdxByClass = adjustSelfSimilarityRankByClass(...
  neighborIdxByClass, classLabels, varargin)
%adjustSelfSimilarityRankByClass - force self-similarity to have rank 1.
% Adjust the rank of a sample's nearest neighbors so that the sample is
% always its own nearest neighbor. This function is suited for use in
% conjunction with findNearNeighborsByRank(). The latter function uses
% Matlab's sort() to sort a sample's nearest neighbors. In case of ties,
% sort() orders the results by keeping the original order of the  tied
% neighbors. This is not always desirable, as we often assume that a
% sample's nearest neighbor is the sample itself.
%
% USAGE: adjustedNeighborIdx = adjustSelfSimilarityRankByClass( ...
%     neighborIdx, classLabels, removeFlag)
% neighborIdx - N-by-1 cell array, in the same format as returned by the
%     function findNearNeighborsByClass().
% classLabels - N-by-1 vector of class labels corresponding to the cells of
%     neighborIdx.
% removeFlag - optional string, can be either 'keep' (default) or 'remove'.
%     If set to 'remove' the self-similar sample is removed from the set
%     neighborhood as are the farthest neighbors from the classes other
%     thatn the sample's. Thus the number of neghbors is reduced by 1 from
%     eah class.
%
% See also findNearNeighborsByClass, adjustSelfSimilarityRank().

% Written by Luca Cazzanti, engineer.
% Applied Physics Laboratory, University of washington, Jan. 2009.
%$Id:$
%$DateTime:$
%$Author:$

% The class label order corresponds to the rows of each matrix in the input
% cell array.
uniqueClassLabels = unique(classLabels); 

% Assumption: the similarity matrix simMat that generated the neighbor indexing
% was square and the maximum possible index in neighborIdxByClass < size(simMat)
nSamples = size(neighborIdxByClass,1);
for iSample = 1:nSamples
  neighborIdxMat = neighborIdxByClass{iSample};
  rowIdx = find( uniqueClassLabels == classLabels(iSample));
try
  candidateRow = neighborIdxMat(rowIdx,:);
catch
  disp(lasterr)
  keyboard
end
  colIdx = find( candidateRow == iSample);
  tmpVec = candidateRow(1:colIdx);
  tmpVec = circshift(tmpVec, [0 1]);
  candidateRow(1:colIdx) = tmpVec;
  neighborIdxMat(rowIdx,:) = candidateRow;
  neighborIdxByClass{iSample} = neighborIdxMat;
end % for iSample

% Should we go ahead and eliminate the self-similarities?
switch nargin
  case 3
    removeFlag = varargin{1};
  case 2 % default case
    removeFlag = 'keep';
  otherwise
    error('Invalid number of inputs')
end % switch

if strcmpi(removeFlag, 'remove') ==  true

  for iSample = 1:nSamples
  
    neighborIdxMat = neighborIdxByClass{iSample};
    % Remove the sample itself
    rowIdx = find( uniqueClassLabels == classLabels(iSample));
%     if isempty(rowIdx)
%       keyboard
%     end
    selfSimilarRow = neighborIdxMat(rowIdx,:);
    selfSimilarRow(1) = [];
    % Remove the farthest of the neighbors from the other classes so that
    % the number of neighbors per class is the same for all classes
    tempMat = neighborIdxMat(:,1:end-1);
    tempMat(rowIdx,:) = selfSimilarRow;
    
    neighborIdxByClass{iSample} = tempMat;

  end % for iSample

elseif ~strcmpi(removeFlag, 'keep')
  error(['Invalid string input: ' removeFlag]);
end % if strcmpi 

return % END adjustSelfSimilarityRankByClass
