function [classEst, posteriors] = classifyBda(testFeatureVec, modelParams)
%classifyBda - classify using the BDA classifier. 
%Classify a test feature vector using the BDA classifier.
% USAGE:
% [classEst, posteriors] = classifyBda(testFeatureVec, modelParams)
%
% testFeatureVec: 1-by-F feature vector to be classified.
% modelParams: the BDA model parameters as computed by computeBdaModels().
% classEst: the class estimate.
% posteriors: the posterior probabilities computed by the BDA classifier.
%
% See also computeBdaModels, classifyLocalSda.

% Written by Luca Cazzanti, enginner.
% Applied Physics Lab - University of Washington, Feb 2009.
% Originally written by Santosh Srivastava and Sergey Feldman,
% University of Washington, Dept. Electrical Engineering.

d = length(testFeatureVec); % number of dimensions
nClasses = length(modelParams.classCovNoNorm); % number of classes
piTerm = -d/2*log(pi); % term constant to all discriminants
BnumError = eye(d); % this is the Bh we will use in case of a numerical error
logClassDiscriminant = zeros(1,nClasses);

for iClass = 1:nClasses
  D = modelParams.classCovNoNorm{iClass} + modelParams.Bh;
  Z = testFeatureVec - modelParams.classMeans(iClass,:);
  const = (modelParams.classCounts(iClass) + 1 )/ modelParams.classCounts(iClass);
  temp = eig(const*D);
  bottomDetTerm = -0.5*sum(log(temp));
  ZZterm = -(modelParams.classCounts(iClass) + modelParams.q + 1)/2 * ...
    log(1 + Z*D^-1*Z'/const);
  logClassDiscriminant(iClass) = modelParams.gammaTerm(iClass) + ...
    bottomDetTerm + ZZterm + piTerm;
  
  % If numerical errors are encountered despite regularization, we will
  % revert to Bh = identity
  if isinf(logClassDiscriminant(iClass)) || ~isreal(logClassDiscriminant(iClass))
    display('switching to Bh = I due to numerical errors')
    D = modelParams.classCovNoNorm{iClass} + BnumError;
    bottomDetTerm = -0.5*sum(log(eig(const*D)));
    ZZterm = -(modelParams.classCounts(iClass)+ q + 1)/2 * ...
      log(1 + Z*D^-1*Z'/const);
    logClassDiscriminant(iClass) = modelParams.gammaTerm(iClass) + ...
      bottomDetTerm + ZZterm + piTerm;
  end
end
posteriors = exp(logClassDiscriminant(:)).* modelParams.classPriors(:);
[notUsed, maxIdx] = max(posteriors);
classEst = modelParams.classLabels(maxIdx);

return