function [classEst, logPosteriors, logMarginals, modelParams] = ...
  classifyClassMtmrLocalSda( testPointSim, neighborSimMat, ...
  neighborClassLabels, centroidType, sigma, eta, nIters, simDomain)
%classifyClassMtmrLocalSda - classify a test point with MTMR local SDA.
% Classifies a test point based on its similarity to a set of other points
% using local similarity discriminant analysis (SDA) and multi-task
% manifold regularization (MTMR), where the regularization is across the
% possible classes.
%
% USAGE
% TO DO - THIS IS COPIED FROM LOCAL SDA
% [classEst, logPosteriors, modelParams, ...
%    sameClassLogPosteriors, sameClassEst, logMarginals] = classifyLocalSda( ...
%    simVec, simMat, classLabels, centroidType, simDomain)
% simVec: 1-by-M vector of test point similarities to each of the set points.
% simMat: M-by-M matrix of pairwise similarities of all set points
% classLabels: M-by-1 vector of class labels of the points in the set.
% centroidType - string denoting the type of centroid to use. See
%     findCentroid() for a list of allowed centroid types.
% simDomain - vector of all possible similarity values. Optional parameter;
%     if it is not provided, it is constructed from simMat.
% classEst - class estimate for the test point corresponding to simVec.
% logPosteriors - log of the posterior probabilites for the classes.
% modelParams - structure with the computed maximum entropy model
%     parameters used for classification. See computeMaxEntModels for
%     details.
% sameClassLogPosteriors - disabled, returns -Inf.
% sameClassEst - disabled, returns 0.
% logMarginals - G-by-G matrix of the log of the class-conditional marginal
%     probabilities, where G is the number of classes. The output
%     logPosteriors is computed from this matrix.
%
% See also classifySda

% Written by Luca Cazzanti, engineer.
% Applied Physics Laboratory, University of washington, August 2010.
%$Id:$
%$DateTime:$
%$Author:$

% Initialize some variables 
uniqueClassLabels = unique(neighborClassLabels);
nClasses = length(uniqueClassLabels);

%% All neighbors are from one class --> classify as that class; done.
if nClasses == 1
  classEst = uniqueClassLabels;
  logPosteriors = 0;
  logMarginals = 0;
  modelParams = initSdaModels; %creates empty/default structure
  modelParams.classPriors = 1;
  modelParams.classCounts = size(neighborSimMat,1);
  modelParams.classLabels = uniqueClassLabels;
  % The other fields are empty
  
  return
end


%% All other cases
% Classify based on regularizing the means across classes, then computing
% the local SDA parameters from the regularized means. This is the core of
% this flavor of SDA-MTMR.

modelParams = computeClassMtmrSimParams(neighborSimMat, neighborClassLabels, ...
  centroidType, simDomain, sigma);

% Vectorize for later - these are used inside all loops
classCountsMat = repmat(modelParams.classCounts(:), 1, nClasses);
sumSimToCentroids = modelParams.meanSim .* classCountsMat;
classCountsMat = classCountsMat(:); 
sumSimToCentroids = sumSimToCentroids(:);

nSigmas = length(sigma);
nEtas = length(eta);

%nIters = 1;
classEst = -1000 * ones(nEtas, nSigmas);
for iSigma = 1:nSigmas
  
  thisClassSim = modelParams.classSim(:,:,iSigma);
  
  classEstTemp = -1000 * ones(nEtas, 1);
  for iEta = 1:nEtas
    
    newMeans = -1000 * ones(nClasses^2);
    thisLambdaVec = -1000 * ones(nClasses^2,1);
    thisGammaVec =  -1000 * ones(nClasses^2,1);
    thisEstErrVec = -1000 * ones(nClasses^2,1);
    
    iterClassSim = thisClassSim;
    for iIter = 1:nIters
      
      classSimDiag = diag(iterClassSim);
      denominator = classCountsMat + ...
        eta(iEta) * (sum(iterClassSim,2) - classSimDiag );

      tildeParam = sumSimToCentroids ./ denominator;
      normSimMat = eta(iEta) * iterClassSim ./ ...
        repmat(denominator, 1, nClasses^2);
      for iDim = 1:nClasses^2
        normSimMat(iDim, iDim) = 0;
      end

      newMeans = ( eye(nClasses^2) - normSimMat ) \ tildeParam;
      newMeans = reshape(newMeans, nClasses, nClasses);
      iterClassSim  = computeRbfSimClassMtmr(newMeans, sigma(iSigma));
      iterClassSim = squeeze(iterClassSim);
      
    end % for iIter
    
    
    %  ... compute local SDA parameters from regularized means:
    % pick either Mex or m-script implementation ...
    
    %  ... Mex is faster but tested only on Linux ...
    nSims = length(simDomain);
    for iMean = 1:nClasses^2
      [thisLambdaVec(iMean), thisGammaVec(iMean), ...
        thisEstErrVec(iMean)] = MexComputeLambda( ...
        newMeans(iMean), simDomain, nSims);
    end % for iMean

    % ... M-file is slower but more portable ...
    %for iMean = 1:nClasses^2;
    %[thisLambdaVec(iMean), thisGammaVec(iMean), ...
    %  thisEstErrVec(iMean)] = computeLambda(newMeans(iMean), simDomain);
    %end %for iMean
    
    % ... classify ...
    modelParams.lambda = reshape(thisLambdaVec, nClasses, nClasses);
    modelParams.gamma = reshape(thisGammaVec, nClasses, nClasses);
    modelParams.meanSim = reshape(newMeans, nClasses, nClasses);
    classEstTemp(iEta) = classifySda(testPointSim, modelParams);
  
  end %for iEta
  
  classEst(:, iSigma) = classEstTemp(:);
  
end % for iSigma

return %END main function

