function [classEst, logPosteriors] = classifyMixtureSda(simMat, modelParams)
%classifyMixtureSda - classify using similarity discriminant analysis.
% Classifies samples based on their similarities to two classes, using
% mixtures similarity discriminat analysis. The model parameters are assumed
% previously computed by the function computeMixtureSdaModels().
% USAGE
% [classEst, logPosteriors] = classifyMixtureSda(simVec, modelParams);
% simMat - N-by-M matrix , where each row contains the similarities of a
%     sample to M other samples.
% modelParams - SDA model parameters as computed by computeMixtureSdaModels().
%     See the help for that function for details on the structure of
%     modelParams.
% classEst - N-by-1 vector of class estimates for each row of simMat.
% logPosteriors - N-by-2 matrix of the log of the class posterior
%     probabilities for each sample.
%
% See computeMixture SdaModels() and computeMixturesMaxEntModels() for
% details on the structure of modelParams. 

% Written by Luca Cazzanti, engineer.
% Applied Physics Laboratory, University of Washington, March 2007.
%$Id:$
%$DateTime:$
%$Author:$

[nData, notUsed] = size(simMat); %#ok, we assume nClasses = 2.


%% Trivial model? --> classify as class with 100% prior proability; done.
if modelParams.isTrivial == true
  theOnlyClassLabelIdx = find(modelParams.classPriors == 1);
  if isempty(theOnlyClassLabelIdx)
    error('A trivial model was provided, but no class prior = 1');
  end
  theOnlyClassLabel = modelParams.classLabels(theOnlyClassLabelIdx);
  classEst = theOnlyClassLabel * ones(nData,1);
  logPosteriors = -Inf * ones( nData, length(modelParams.classPriors) );
  logPosteriors(:, theOnlyClassLabelIdx) = 0;
  return
end

%% Non-trivial model

classEst = zeros(nData, 1);
logPosteriors = -Inf * ones( nData,length(modelParams.classPriors) );
classPriors = modelParams.classPriors(:);
nClasses = length(modelParams.classPriors);
for iData = 1:nData
  simVec = simMat(iData, modelParams.centroidIdx);
  
  % THIS COMMENT IS NOT QUITE RIGHT. Whwere are the parameters adjusted below?
  % Also, local SDA now handles this case more consistently, and this
  % function should do the same. REVIEW THIS - LGC Jan 2009
  % THE COMMENT WAS: If the test point similarity is either max or min of
  % similarity domain, its probability is 1: correct the parameters to
  % reflect this.
  
  thisSampleLogPosteriors = zeros(nClasses, nClasses);
  for iClass = 1:nClasses
    tempLambda = modelParams.lambda{iClass};
    tempGamma = modelParams.gamma{iClass};
    thisClassCentroidIdx = ( modelParams.centroidClassLabels == ...
      modelParams.classLabels(iClass) );
    thisSimVec = simVec(thisClassCentroidIdx);
    thisSimVec = [thisSimVec;thisSimVec];
    
    exponentMat = tempLambda .* thisSimVec; 
    probMat = tempGamma .* exp(exponentMat);
    centroidWeights = modelParams.centroidWeights{iClass};
    componentPriors = modelParams.componentPriors{iClass};
    weightedProbMat = probMat .* centroidWeights .* componentPriors;
    normFactor = sum(weightedProbMat,2);
    responsibility = weightedProbMat ./ repmat(normFactor, 1, size(weightedProbMat,2));
    totalClassLogLike = log(probMat + eps) .* responsibility + ...
      log(centroidWeights+eps) .* responsibility;
    totalClassLogLike = sum(totalClassLogLike,2);
    thisSampleLogPosteriors(:,iClass) = log(classPriors(:)) + totalClassLogLike;

  end
  
  % FOR DEBUGGING: use sameClassLogPosteriors
  % FOR NOT DEBUG: use  totalLogPosteriors
  totalLogPosteriors = sum(thisSampleLogPosteriors,2); % this is the discriminant
  %sameClassLogPosteriors = diag(thisSampleLogPosteriors);
  % END DEBUG
  
    % Do both classes have 0 probability? --> Use nearest centroid
  if all(totalLogPosteriors == -Inf)
    classEst(iData) = classifyMultiNearestCentroid(simMat(iData,:), modelParams);
  else
    [notUsed, thisClassEstIdx] = max(totalLogPosteriors); %#ok
    classEst(iData) = modelParams.classLabels(thisClassEstIdx);
  end % if all(thiLogPosteriors == -Inf)
  
  
  % FOR DEBUGGING: use sameClassLogPosteriors
  % FOR NOT DEBUG: use totalLogPosteriors
  %logPosteriors(iData,:) = sameClassLogPosteriors(:).';
  logPosteriors(iData,:) = totalLogPosteriors(:).';
  
  
end % for iData

return
