function classEst = classifyNearestCentroid(dataMat, modelParams, varargin)
%classifyNearestCentroid - classify using nearest centroid.
% Classifies data using a nearest centroid classifier and
% previously-computed model parameters.
%
% USAGE
% classEst = classifyNearestCentroid(dataMat, modelParams, simType)
% dataMat: N-by-M matrix of similarities of N test samples to M training
%   samples.
% modelParams: structure containing the appropirate model parameters as
%   computed by computeCentroidModels().
% simType: optional string, either 'similarity' (default) or
%   'dissimilarity'.
% classEst: N-by-1 vector of estimated class labels; the i-th number
%     corresponds to the most likely class for the i-th row of dataMat.
%
% See also initCentroidModels, computeCentroidModels.

% Written by Luca Cazzanti, engineer
% Applied Physics Laboratory, University of Washington, Jan 2007.
%$Id:$
%$DateTime:$
%$Author:$

[nData, notUsed] = size(dataMat); %#ok % nData used later on

%% Trivial model --> Only one class; return that class as the estimate.
% REMEMBER: if we decide to estimate the priors with laplace correction in
% computeCentroidModels(), this method of identifying the one and only
% class label does not work
if modelParams.isTrivial == true
  theOnlyClassIdx = find(modelParams.classPriors == 1);
  if isempty(theOnlyClassIdx)
    error('A trivial model was provided, but no class prior = 1');
  end
  theOnlyClassLabel = modelParams.classLabels(theOnlyClassIdx);
  classEst = theOnlyClassLabel * ones(nData,1);
  return
end

%% Non-trivial model: pick the class of the nearest centroid
% Make sure the meaning of "nearest centroid" is consistent with the type
% of similarity used. This determines the sorting order of the centroid
% similarities: descending for similarities, ascending for dissimilarities.
% For now, support only 'maxSumSim' centroid type, which means
% similarities, which means  descending order of similarity.
if ~strcmp(modelParams.centroidType, 'maxSumSim')
  error([modelParams.centroidType ' is not a supported centroid type'])
end
simMat = dataMat(:,modelParams.centroidIdx);
nCentroids = length(modelParams.centroidIdx);
switch nargin
  case 2
    simType = 'similarity';
  case 3
    simType = varargin{1};
  otherwise
    error('Wrong number of input arguments')
end % switch nargin
[sortedSimIdx, sortedSimMat] = findNearNeighbors(simMat, nCentroids, ...
  simType);

% Break ties according to the class priors. Works like this:
% a) find the rows for which there was at least one tie for maximum
%    similarity to the class centroids.
% b) for those rows, reorder the tied similarities so that they are in
%    decreasing order of their corresponding class prior probabilities.
maxSimIdx = sortedSimIdx(:,1);
classPriors = modelParams.classPriors(:).';
classPriorsSortedBySim = classPriors(sortedSimIdx);
rowTieIdx = find(sortedSimMat(:,1) == sortedSimMat(:,2));
for rowIdx = rowTieIdx(:).';
  colIdx = find(sortedSimMat(rowIdx,:) == sortedSimMat(rowIdx,1));
  [notUsed, maxPriorIdx] = max(classPriorsSortedBySim(rowIdx,colIdx), ...
    [], 2); %#ok
  maxSimIdx(rowIdx) = sortedSimIdx(rowIdx, colIdx(maxPriorIdx));
end

% Classify - the first centroid index determines the class estimate.
classEst = modelParams.classLabels(maxSimIdx);

return