function [classEst, logPosteriors, modelParams, nonRegModelParams] = ...
  classifyRegularizedMeanLocalSda(testPointSim, neighborSimMat, ...
  neighborClassLabels, regularizingParams)
%classifyRegularizedMeanLocalSda - mean-regularized local SDA classifier. 
% Classifies a test sample using the mean-regularized local SDA classifier.
% USAGE
% [classEst, logPosteriors, modelParams, nonRegModelParams] = ...
%    classifyRegularizedMeanLocalSda(simVec, simMat, classLabels,...
%    regularizingParams)
% simVec: 1-by-M vector of test point similarities to each of the set points.
% simMat: M-by-M matrix of pairwise similarities of all set points
% classLabels: M-by-1 vector of class labels of the points in the set.
% regularizingParams: structure containing several parameters needed for
%    regularization and model estimation. Use
%    createRegularizedMeanParamStruct() to create this structure/
% classEst - class estimate for the test point corresponding to simVec.
% logPosteriors - log of the posterior probabilites for the classes.
% modelParams - structure with the computed maximum entropy model
%     parameters used for classification. See computeMaxEntModels for
%     details.
% nonRegModelParams - structure with the non-regularized local SDA model
%     parameters, as returned by computeMaxEntModels. 
%
% The function regularizes the class-conditional mean similarity of the
% local SDA model in two ways. If regularizingParams.meanType is 'average',
% it uses the average of the local SDA mean similarities as the
% regularizing values. In this case the fields regularizingParams.meanSim
% and regularizingParams.classLabels are ignored. If
% regularizingParams.meanType is 'global', it regularizes using the
% parameters regularizingParams.meanSim, which are assumed to have
% been computed by computeSdaModels(). The new mean similarity values are
% then used as inputs to computeLambda to re-compute the local SDA models
% using the updated mean similarities.
%
% See also createRegularizedMeanParamStruct.

% Written by Luca Cazzanti, engineer.
% Applied Physics Laboratory, University of Washington, March 2009.
%$Id:$
%$DateTime:$
%$Author:$

%% Assign some parameters
meanType = regularizingParams.meanType;
regParam = regularizingParams.regParam;
regMean = regularizingParams.regMean;
regClassLabels = regularizingParams.classLabels;
simDomain = regularizingParams.simDomain;
centroidType = regularizingParams.centroidType;

%% Compute parameters for non-regularized local SDA
nonRegModelParams = computeMaxEntModels(neighborSimMat, neighborClassLabels, ...
  centroidType, simDomain);
nLocalClasses = length(nonRegModelParams.classLabels);

%% Which regularizing lambda are we using?
switch lower(meanType)
  
  % use the average of the local SDA class-conditional means
  case 'average'

    newRegMean = mean(nonRegModelParams.meanSim(:));
    
    
  % use the class-conditional means provided as input
  case 'global' 
  
  % Must line up the input means with the local means matrix because the
  % local model might have less classes than the provided model.
  idxVec = [];
  for iClass = 1:nLocalClasses
    idx = find(regClassLabels == nonRegModelParams.classLabels(iClass));
    idxVec = [idxVec idx];
  end
  newRegMean = regMean(idxVec,:);
  newRegMean = newRegMean(:,idxVec);
  
  otherwise 
    error(['Unsupported mean-regularization type: ' meanType]);
    
end %switch

%% Re-compute the local SDA model params using the regularized means
modelParams = nonRegModelParams;

% Here's the new class-conditional mean similarity to use
meanSim = ( 1 - regParam) * modelParams.meanSim + ...
  regParam * newRegMean;
modelParams.meanSim = meanSim;
maxSim = max(simDomain);
minSim = min(simDomain);

% Compute the local SDA model parameters using the new mean similarity
for gClass = 1:size(modelParams.gamma, 1)
  for hClass = 1:size(modelParams.gamma, 2)
    
    tempMeanSim = modelParams.meanSim(gClass, hClass);
    if (tempMeanSim == maxSim || tempMeanSim == minSim)
      tempLambda = -Inf;
      tempGamma = 1; % either case
      tempEstErr = 0; % either case
    else
      [tempLambda, tempGamma, tempEstErr] = computeLambda(...
        tempMeanSim, simDomain);
    end % if
    
    modelParams.lambda(gClass, hClass) = tempLambda;
    modelParams.gamma(gClass, hClass) = tempGamma;
    modelParams.estErr(gClass, hClass) = tempEstErr;
    
  end % for hClass 
end % for gClass

%% Classify using the regularized model parameters
[classEst, logPosteriors] = classifySda(testPointSim, modelParams);

return
  