function [classEst, logPosteriors, sameClassEst, ...
  sameClassLogPosteriors, logMarginals] = classifySda(simMat, modelParams)
%classifySda - classify using similarity discriminant analysis.
% Classifies a point based on its similarity to class centroids, using
% similarity discriminat analysis. The model parameters are assumed
% previously computed by the function computeSdaModels().
%
% USAGE
% [classEst, logPosteriors, sameClassEst, sameClassLogPosteriors] = ...
%    classifySda(simMat, modelParams)
% simMat - N-by-M matrix, where each row contains the similarities of a
%     test sample to M other samples.
% modelParams - SDA model parameters as computed by computeSdaModels(). See
%     the help for that function for details on the structure of
%     modelParams.
% classEst - N-by-1 vector of class estimates for each row of simMat.
% logPosteriors - N-by-G matrix of the log of the class posterior
%     probabilities for each test sample.
% sameClassEst - (disabled) N-by-1 vector of class estimates for each row of simMat,
%     obtained by considering only the same-class SDA model.
% sameClassLogPosteriors (disabled) - N-by-G of the log of the class posterior
%     probabilities for each test sample, obtained from the same-class SDA
%     model.
% logMarginals - N-by-1 cell array. Each element is a G-by-G matrix of the
%     log of class-conditional marginal probabilities, where G is the
%     number of classes. The output logPosteriors is computed from this matrix.
%
% See computeSdaModels() and computeMaxEntModels() for details on the
% structure of modelParams. 

% Written by Luca Cazzanti, engineer.
% Applied Physics Laboratory, University of Washington, November 2006.
%$Id:$
%$DateTime:$
%$Author:$
%
%
% Revised April 13, 2007 by Maya Gupta to do multiclass.

% NOTE: if the same-class SDA performs well, we should put it in its own
% dedicated function. Right now same-class SDA is experimental and its
% implementation is inefficient: we compute all cross-class SDA models and
% pull out the diagonal from the log-posterior matrix.
% LGC: Nov 16, 2009
% Update: cross-class SDA works at least as well as, and sometimes better
% than, same-class SDA, so we won't pursue same-class SDA any longer for now.
% Code for computing same-class SDA is still in this script at the end of
% the for loop, but it's commented out. We return bogus values for speed.
% LGC Feb 23, 2010

[nData, notused] = size(simMat); %#ok
%sameClassLogPosteriors = zeros(nData,1); % same-class fucntionality
%disabled


%% Trivial model? --> classify as class with 100% prior proability; done.
if modelParams.isTrivial == true
  theOnlyClassLabelIdx = find(modelParams.classPriors == 1);
  if isempty(theOnlyClassLabelIdx)
    error('A trivial model was provided, but no class prior = 1');
  end
  theOnlyClassLabel = modelParams.classlabels(theOnlyClassLabelIdx);
  classEst = theOnlyClassLabel * ones(nData,1);
  logPosteriors = -Inf * ones( nData, length(modelParams.classPriors) );
  logPosteriors(:, theOnlyClassLabelIdx) = 0;
  sameClassLogPosteriors = logPosteriors;
  sameClassEst = classEst;
  return
end

%% Non-trivial model
nClass = length(modelParams.classPriors);
classEst = zeros(nData, 1);
sameClassEst = zeros(nData, 1);
logPosteriors = -Inf * ones( nData,length(modelParams.classPriors) );
sameClassLogPosteriors = -Inf * ones( nData,length(modelParams.classPriors) );
classPriors = modelParams.classPriors(:);

%minSim = min(modelParams.simDomain);
%maxSim = max(modelParams.simDomain);

% These are used to handle the case meanSim = maxSim = minSim in the loop
% THIS SHOULD BE isinf(modelParams.lambda)! WHY DID I DO IT THIS WAY?
infiniteLambdaIdx = modelParams.lambda == -Inf; %logical matrix
applyCorrection = any(infiniteLambdaIdx(:));
logMarginals = cell(nData, 1);
for iData = 1:nData % iterate through test points
  % get sim between test point and each class centroid
  testToCentroidSimVec = simMat(iData,modelParams.centroidIdx);
    
  % LGC: 12/31/2008 THIS IS INCORRECT! SEE BELOW FOR FIX
  % If the test point similarity is either max or min of similarity domain,
  % its probability is 1: correct the parameters to reflect this.
  %
  %extremeIdx = find( (tCMat == minSim) + (tCMat == maxSim) );
  %tempLambda = modelParams.lambda;
  %tempLambda(extremeIdx) = 0; %#ok; log(1) = 0; we use log-distriminant
  %tempGamma = modelParams.gamma;
  %tempGamma(extremeIdx) = 1;
  %
  %END OF INCORRECT CODE 
  
  % LGC 01-02-2009: THIS IS THE CORRECT WAY TO HANDLE IT
  % Check if lambda == -Inf: if yes, then the pmf is a discrete delta
  % function centered at meanSim. Manually set 
  % P(sim(x, mu_h) = meanSim_gh | Y=g ) = 1 if the test similarity is equal
  % to the mean similarity. The class-conditional probability of all other
  % similarity values are automatically set to 0 due to -Inf in exponent.
  tempLambda = modelParams.lambda;
  tempGamma = modelParams.gamma;
  %if applyCorrection == true
  %  meanSimIdx = logical(sign(abs(modelParams.meanSim - tCMat))); 
  %  degenerateIdx = infiniteLambdaIdx & meanSimIdx; 
  %  tempLambda(degenerateIdx) = 0; %#ok, log(1) = 0; we use log-discriminant
  %  tempGamma(degenerateIdx) = 1;
  %end
  
  % Moved the above code to a separate function: 
  if applyCorrection ==  true
    [tempLambda, tempGamma] = applyMaxEntParameterCorrection(tempLambda, ...
      tempGamma, modelParams.meanSim, testToCentroidSimVec);
  end
  % END CORRECT WAY OF HANDLING EXCEPTION
  
  % Here's the classification formula
  tCMat = repmat(testToCentroidSimVec, nClass,1); 
  exponentMat = (tempLambda .* tCMat);
  % CAREFUL: if exponentMat contains -Inf and tCMat contains 0 in
  % corresponding elements the result of the product is -NaN. This does not
  % affect the classification because NaN is ignored by max(), but be aware
  % of this fact. LGC, Jan 21, 2009
  % Decided to change this to account for NaNs
  nanProblemIdx = isnan(exponentMat); % NaN correction
  exponentMat(nanProblemIdx) = -Inf; % NaN correction
  logDiscriminant = exponentMat + log(tempGamma);
  thisLogPosteriors = sum(logDiscriminant,2) + log(classPriors);
    
  % Do all classes have 0 probability? Then use nearest centroid to
  % classify
  if all( isinf(thisLogPosteriors) )
    classEst(iData) = classifyNearestCentroid(simMat(iData,:), modelParams);
  else
    [notUsed, thisClassEstIdx] = max(thisLogPosteriors);
    classEst(iData) = modelParams.classLabels(thisClassEstIdx);
  end % if all( isinf(thisLogPosteriors) )
  
  logPosteriors(iData,:) = thisLogPosteriors;
  logMarginals{iData} = logDiscriminant;
  
  %%% SAME-CLASS SDA
  % Same-class SDA works well, but not well enough to replace the original
  % cross-class SDA, so we take it out.
  if(0)
  % Classify using only the intra-class SDA models
  sameClassLogPosteriors(iData,:) = diag(logDiscriminant) + log(classPriors);
  
  % Do all classes have 0 probability? Then use nearest centroid to
  % classify
  if all( isinf(sameClassLogPosteriors(iData,:) ) )
    sameClassEst(iData) = classifyNearestCentroid(simMat(iData,:), modelParams);
  else
    [notUsed, thisClassEstIdxSameClass] = max(sameClassLogPosteriors(iData,:)); 
    sameClassEst(iData) = modelParams.classLabels(thisClassEstIdxSameClass);
  end % if all(isinf( sameClassLogPosteriors(iData,:) ) )
  end %if(0)
  %%%
  
end % for iData

return