function [classEst, logPosteriors] = classifySdaNNSim(simMat, ...
  classLabels, modelParams)
%classifySdaNNSim - nearest neighbor similarity SDA classifier.
% Classifies test saples based on their similarities to their respective
% k-th nearest neghbor. 
%
% USAGE
% [classEst, logPosteriors] = classifySdaNNSim(simMat, classLabels, modelParams);
% simMat - N-by-M matrix , where each row contains the similarities of a
%     sample to M other samples.
% modelParams - SDA model parameters as computed by computeMaxEntModelsNNSim().
%     See the help for that function for details on the structure of
%     modelParams.
% classEst - N-by-1 vector of class estimates for each row of simMat.
% logPosteriors - N-by-C matrix of the log of the class posterior
%     probabilities for each sample, where C is the number of classes
%
% See also computeMaxEntModelsNNSim().

% Written by Luca Cazzanti, engineer.
% Applied Physics Laboratory, University of Washington, November 2006.
%$Id:$
%$DateTime:$
%$Author:$

[nData, notUsed] = size(simMat); %#ok

%% Trivial model? --> classify as class with 100% prior proability; done.
if modelParams.isTrivial == true
  theOnlyClassLabelIdx = find(modelParams.classPriors == 1);
  if isempty(theOnlyClassLabelIdx)
    error('A trivial model was provided, but no class prior = 1');
  end
  theOnlyClassLabel = modelParams.classlabels(theOnlyClassLabelIdx);
  classEst = theOnlyClassLabel * ones(nData,1);
  logPosteriors = -Inf * ones( nData, length(modelParams.classPriors) );
  logPosteriors(:, theOnlyClassLabelIdx) = 0;
  return
end

%% Non-trivial model
classEst = zeros(nData, 1);
logPosteriors = -Inf * ones( nData,length(modelParams.classPriors) );
classPriors = modelParams.classPriors(:);

% ... reorder columns according to nearest-neighbor ...
[nData, maxNeighbors] = size(simMat);
neighborIdx = findNearNeighbors(simMat, maxNeighbors, 'similarity');
classLabelMat = zeros(nData, maxNeighbors);
for iRow = 1:nData
  simMat(iRow,:) = simMat(iRow, neighborIdx(iRow,:));
  classLabelMat(iRow,:) = classLabels(neighborIdx(iRow,:));
end


% ... find each sample's nearest neighbor from each class: this amounts to
% finding the first occurrence of each class label in the neighbors' class
% label vector ...
nClasses = length(modelParams.classLabels);
simToNearestNeighbors = zeros(nData, nClasses);
nearestNeighborsIdx = zeros(nData, nClasses);
nearestNeighborsLabels = zeros(nData, nClasses);
representativeNeighbor = modelParams.representativeNeighbor;
for iRow = 1:nData
  for iLabel = 1:nClasses
    idx = find(classLabelMat(iRow,:) == modelParams.classLabels(iLabel));
    actualNeighbor = min(representativeNeighbor, length(idx));
    simToNearestNeighbors(iRow,iLabel) = simMat(iRow, idx(actualNeighbor));
    nearestNeighborsIdx(iRow, iLabel) = neighborIdx(iRow,idx(actualNeighbor));
    nearestNeighborsLabels(iRow, iLabel) = classLabelMat(iRow, idx(actualNeighbor));
  end % for iLabel
end % for iRow

%keyboard

% classify each sample
for iData = 1:nData
  simVec = simToNearestNeighbors(iData, :);
  
  % Apply the model correction - NEW WAY
  [newLambda newGamma] = applyMaxEntParameterCorrection(modelParams.lambda, ...
  modelParams.gamma, modelParams.meanSim, simVec);
  
  simVec = repmat(simVec, nClasses,1); 
  
  % Here's the classification formula
  exponentMat = (newLambda .* simVec);
  
  % Careful: 0 * -Inf = NaN; in the rare case the simVec(i,j) = 0 and
  % lambda(i,j) = 0 results will be screwed up. Force it to -Inf; code 
  % below will take care of it. 
  exponentMat(isnan(exponentMat)) = -Inf;

    %OLD WAY 
  % Was there a degenerate pmf such that:
  % lambda = -Inf and similarity = meanSim? If yes, we must change the
  % class-conditional probability to 1 for that similarity value.
  
  %infiniteLambdaIdx = modelParams.lambda == -Inf; %logical matrix
  %meanSimIdx = sign(abs(modelParams.meanSim - simVec)); %logical matrix
  %degenerateIdx =  logical(infiniteLambdaIdx .* meanSimIdx) ; % logical AND
  %exponentMat(degenerateIdx) = 0; %#ok, log(1) = 0; we use log-discriminant
  
  % Compute log-posteriors
  logDiscriminant = exponentMat + log(newGamma);
  thisLogPosteriors = sum(logDiscriminant,2) + log(classPriors);
  
  % Do all classes have 0 probability? Then use nearest neighbor to
  % classify
  if all(thisLogPosteriors == -Inf)
    classEst(iData) = nearestNeighborsLabels(iData,1);
  else
    [notUsed, thisClassEstIdx] = max(thisLogPosteriors); %#ok
    classEst(iData) = nearestNeighborsLabels(iData,thisClassEstIdx);
  end % if all(thiLogPosteriors == -Inf)
  
  logPosteriors(iData,:) = thisLogPosteriors;
  
end % for iData

return