function modelParams = computeCentroidModels(simMat, classLabels, ...
  centroidType)
%computeCentroidModels - compute the class centroids from similarity data.
% Computes the indeces of the class centroids based on pairwise similarity
% data; also returns ancillary information such as class prior probabilities,
% and class labels.
%
% USAGE
% modelParams = computeCentroidModels(simMat, classLabels, centroidType);
% simMat: N-by-N matrix of pairwise similarities.
% classLabels: N-by-1 vector of class labels for each row of simMat.
% centroidType - string indicating the desired type of centroid. See
%   findCentroid() for a list of supported centroid types.
% modelParams: structure containing class centroid indeces into simMat and
%   ancillary information. The fields are:
%   * isTrivial: boolean flag, true if only one class is represented in the
%     training data.
%   * classPriors: C-by-1 vector of class prior probabilities, where C is
%     the number of classes represented in classLabels. 
%   * classCounts: C-by-1 vector of class frequency counts for the training
%     set.
%   * classLabels: C-by-1 vector of unique class labels.
%   * centroidIdx: C-by-1 vector of indeces into the rows of simMat,
%     indicating the class centroids.
%   * centroidType: string denoting the type of centroid used for the
%     models.
% 
% See also initCentroidModel, classifyNearestCentroid, findCentroid.

% Written by Luca Cazzanti, engineer
% Applied Physics Laboratory, University of Washington, Jan 2007.
%$Id:$
%$DateTime:$
%$Author:$

% Initialize to empty model structure
modelParams = initCentroidModels;

% Compute model parameters
uniqueClassLabels = unique(classLabels);
nClasses = length(uniqueClassLabels);

% If only one class is observed, then the model is trivial and the
% classifier will always classify as this class. The classifier will need to
% know this by looking at the isTrivial field.
if nClasses == 1
  modelParams.isTrivial = true;
else
  modelParams.isTrivial = false;
end

% The following works for arbitrary number of classes
centroidIdx = zeros(nClasses, 1);
classCounts = zeros(nClasses, 1);
for iClass = 1:nClasses
  thisClassLabel = uniqueClassLabels(iClass);
  thisClassIdx = classLabels == thisClassLabel;
  thisClassCentroidIdx = findCentroid( simMat(thisClassIdx, thisClassIdx), ...
    centroidType);
  
  % Convert: index into class submatrix --> index into entire simMat. 
  tmpIdx = find(thisClassIdx);
  centroidIdx(iClass) = tmpIdx(thisClassCentroidIdx);
  
  % Count number of samples in each class
  classCounts(iClass) = sum(thisClassIdx);
end %for iClasses

% Put parameters in result structure
modelParams.centroidIdx = centroidIdx;
modelParams.classLabels = uniqueClassLabels;
modelParams.centroidType = centroidType;
modelParams.classCounts = classCounts; 
modelParams.classPriors = classCounts / sum(classCounts);

% TO DO: should we apply Laplace correction to estimate of priors?

return