function [lambda, gamma, estErr] = computeLambda(classMean, domainVector)
%computeLambda - optimize exponent in MaxEnt exponential distribution.
% Uses the Matlab built-in function fminsearch() to find the optimum value
% of lambda for a mean-constrained maximum entropy problem.
% USAGE
% [lambda, gamma, estErr] = computeLambda(classMean, domainVector)
% classMean - mean constraint in MaxEnt problem.
% domainVector - vector of possible (discrete) values taken on by the
%     exponent. 
% lambda - value of the optimized exponent.
% gamma - values of normalization constant.
% estErr - estimation error, abs(estimatedClassMean - classMean).

% Written by Maya Gupta, Dept EE, University of Washington.
% Modified by Luca Cazzanti, APL-UW.

oneRowVec = ones(1,length(domainVector));
[lambda, estErr] = fminsearch(@optLambda, 0);

% This is a little faster than sum()
expValue = oneRowVec * exp(lambda * domainVector);

tolerance = 10*eps;
if expValue < tolerance
  lambda = -Inf;
  gamma = 1;
else
  gamma = 1/ ( oneRowVec * exp(lambda * domainVector) ); % normalization factor
end

function theErr = optLambda(posLambda)
  elambdaVect = exp(posLambda*domainVector);
  %sumELambdaVect = sum(elambdaVect);
  %sumELambdaVect = row_sum(elambdaVect);
  sumELambdaVect = oneRowVec * elambdaVect;
  
  tolerance = 10*eps;
  if sumELambdaVect < tolerance
    posLambda = -Inf; 
    theErr = 0;
    return
  end
    
 % normalize the pmf and then calc the mean:
 meanpmf = oneRowVec * ( domainVector .* (elambdaVect/sumELambdaVect) );
 theErr = abs(meanpmf - classMean); % abs error to empirical mean

end % end optLambda

end % end computeLambda
