function modelParams = computeMtmrSimParams( simMat, classLabels, ...
  centroidType, simDomain, sigma)
%computeMaxEntModels - compute the MTMR parameters based on similarities.
% Computes the multi-task manifold regularization parameters needed for
% MTMR SDA.
%
% USAGE
% TO DO - THIS WAS COPIED FROM OTHER FUNCTION
% modelParams = computeMaxEntModels(simMat, classLabels, centroidType, ...
%     simDomain)
% siMat: N-by-N matrix of pairwise similarities.
% classLabels: N-by-1 vactor of class labels corresponding to each row of
%     simMat. 
% centroidType: string denoting the type of centroid to use. See
%     findCentroid() for a list of supported centroid types.
% simDomain: optional vector of possible similarity values. If not
%     provided, it is computed by taking the unique values of simMat. 
% modelParams: structure containing the model parameters. The fields are:
%     * isTrivial: boolean flag, true if only one class is represented in
%     the training data.
%     * classPriors: C-by-1 vector of class prior probabilities, where C is
%     the number of classes represented in classLabels. 
%     * classCounts: C-by-1 vector of class frequency counts for the
%     training set.
%     * classLabels: C-by-1 vector of unique class labels.
%     * centroidIdx: 1-by-C vector containing the indices into the 
%     columns of simMat corresponding to the class centroids.
%     * centroidType: string denoting the type of centroid used for the
%     models.
%     * lambda: C-by-C matrix of exponent parameters, ordered as
%     [lambda11 lambda12 ... lambda1C;
%      lamda21 lamda22 ... lambda1C; 
%      ...
%      lambdaC1 lambda C2 ... lambdaCC],
%      where lambda11 is the exponent in the maximum entropy distribution
%      of the similarities to class 1 assuming that a point is in class1;
%      lambda 12 is the exponent in the maximum entropy distribution of the
%      similarities to class 2 given that a point is in class 1; etc.
%     * gamma: C-by-C matrix of normalization constants, arranged the same
%     way as the lambda matrix.
%     * meanSim: C-by-C matrix of mean similarities, arranged as the same
%     way as the lambda matrix, where meanSim11 is the
%     average similarity of the samples in class 1 to the centroid of class
%     1; meanSim12 is the average similarity of the samples in class 1 to
%     the centroid of class 2; etc.
%     * estErr: C-by-C matrix containing the estimation errors as returned
%     by fmisearch(), arranged the same way as the matrix lambda.
%     * simDomain: vector of possible similarity values.
%
% See also initSdaModels, computeSdaModels, classifySda.

% Written by Luca Cazzanti, engineer.
% Applied Physics Laboratory, University of Washington, November 2006.
%
% This multi-class code is a revised version of the two-class code.
% Revisions done by Maya Gupta, Dept. of EE, Univ. of Washington
% April 13, 2007.

%% Quick check on number of classes
uniqueClassLabels = unique(classLabels);
nClasses = length(uniqueClassLabels);

if(0)
%% Get similarity domain vector
% Is the similarity domain provided or should it be inferred from the
% similarity matrix?
switch nargin
  case 4 % provided 
    similarityDomainVector = varargin{1};
  case 3 % compute
    similarityDomainVector = unique(unique(simMat));
  otherwise
    error('Wrong number of input arguments');
end % case
end %if(0)

% These are used later
maxSim = max(simDomain);
minSim = min(simDomain);
nSims = length(simDomain);

% Mark each class's data.
% The g-th row of classIdx marks those samples of the g-th class
% with a "1", and all other samples with a "0"
classIdx = logical(zeros(nClasses, length(classLabels))); % pre-allocated
for g=1:nClasses % iterate through classes grabbing ID lists
  classIdx(g, :) = classLabels == uniqueClassLabels(g);
end

%% GET MODEL PARAMETERS AND STORE THEM
% REVIEW THIS TEXT - NOT RELEVANT TO MTMR RIGHT NOW
% Compute centroids, mean similarity to centroids, and lambdas for the
% class models.
% IMPORTANT:
% If the class-conditional mean is equal to the maximum or the minimum of the
% similarity domain (this can happen for local SDA), then there is only one
% possible value for the similarity, that is the mean value = max = min. In
% this case computeLambda() gives a garbage result for lambda. The correct
% result would be lambda = -Inf, but this has the problem that *all*
% similarities have 0 class-conditional probability. Instead, the correct 
% pmf would be a discrete delta function centered around the mean
% similarity. For this degenerate case, we assign lambda = -Inf here, and 
% correct the pmf to be the appropriate delta function in classifySda().

%% Put results in structure
modelParams = initSdaModels; %initialize to empty fields
modelParams.isTrivial = false;

% Next there are two for-loops to get the model parameters:
% the outer loop on h specifies the class centroid
% and the inner loop on g specifies which class you are conditioning on

% Compute the centroids, mean sims to class h centroid, and the similarity
% between classes  
for h=1:nClasses 
    
  %----------- get class h centroid ----------------%
  tempSimMat = simMat(classIdx(h,:), classIdx(h,:));
  tempCentroidId = findCentroid(tempSimMat, centroidType);
  % tempCentroidId is the index into the class-h class list
  % here's where the h-class samples are in the full list:
  hClassList = find(classLabels == uniqueClassLabels(h));
  % so map to the full-list index for the centroid
  tempCentroidIdinFullList = hClassList(tempCentroidId);
  % store full simMat's centroid's index:
  modelParams.centroidIdx(h) = tempCentroidIdinFullList;
  %----------- end get class h centroid --------------%
  
  % Condition on test point being from class g
  for g=1:nClasses 
    
    % Compute the mean similarity to centroid of class h ...
    tempSimMat = simMat(classIdx(g,:), classIdx(h,:));
    
    %  ... use the class-h-list index for the class-h centroid
    tempMeanSim= mean(tempSimMat(:, tempCentroidId));
    modelParams.meanSim(g,h) = tempMeanSim;
    
  end % for: condition on x in class g
end %for: class h centroid

% Compute similarity between classes
% Here we choose the RBF kernel on the mean similarities

[modelParams.classSim, modelParams.diffSim] = ...
  local_computeRbfClassSim(modelParams.meanSim, sigma);

% Store ancillary information too. Note the Laplace correction for the
% priors.
modelParams.classCounts = sum(classIdx'); % store # of each class
modelParams.classPriors = ( modelParams.classCounts + 1 )/ ...
  ( sum(modelParams.classCounts) + nClasses);
modelParams.classLabels = uniqueClassLabels;
modelParams.simDomain = simDomain;
modelParams.centroidType = centroidType;
modelParams.sigma = sigma;

return %END main function

%% LOCAL FUNCTIONS

% local_computeRbfClassSim()
function [classSimMat, diffSimMat] = local_computeRbfClassSim(meanSimMat, sigma)
nClasses = size(meanSimMat,1);
diffSimMatSize = nClasses^2;
diffSimMat = zeros(diffSimMatSize);

meanSimMat = meanSimMat(:);
for iSim = 1:diffSimMatSize
  
  for jSim = (iSim+1):diffSimMatSize
    diffSimMat(iSim, jSim) = meanSimMat(iSim) - meanSimMat(jSim);
    % Matrix of differences is anti-symmetric by definition
    diffSimMat(jSim, iSim) = -diffSimMat(iSim, jSim);
  end 
  
end

% Similarity between classes is the RBF kernel of the difference between
% mean similarities
classSimMat = exp( -diffSimMat.^2 /sigma );

return
% END local_computeRbfClassSim()
