function modelParams = computeMultiCentroidModels(simMat, classLabels, ...
  initCentroidIdx, centroidType)
%computeMultiCentroidModels - compute multiple class centroids from similarity.
% Computes the indeces of multiple class centroids based on pairwise
% similarities, using the similarity-based k-medoid algorithm in
% findMultipleCentroids(). It also returns ancillary information such as
% class prior probabilities, and class labels. 
%
% USAGE
% modelParams = computeMultiCentroidModels(simMat, classLabels, centroidType);
% simMat: N-by-N matrix of pairwise similarities.
% classLabels: N-by-1 vector of class labels for each row of simMat.
% initCentroidIdx: M-by-1 vector of initial centroids indices. 
% centroidType - string indicating the desired type of centroid. See
%   findMultipleCentroid() for a list of supported centroid types.
% modelParams: structure containing class centroid indeces into simMat and
%   ancillary information. The fields are:
%   * isTrivial: boolean flag, true if only one class is represented in the
%     training data.
%   * classPriors: C-by-1 vector of class prior probabilities, where C is
%     the number of unique classes in classLabels. 
%   * classCounts: C-by-1 vector of class frequency counts for the training
%     set.
%   * classLabels: C-by-1 vector of unique class labels.
%   * centroidIdx: M-by-1 vector of indeces into the columns of simMat,
%     indicating the class centroids.
%   * centroidClassLabels: M-by-1 vector of class labels corresponding to
%     centroidIdx.
%   * centroidType: string denoting the type of centroid used for the
%     models.
% 
% See also classifyMultiNearestCentroid, findMultipleCentroids.

% Written by Luca Cazzanti, engineer
% Applied Physics Laboratory, University of Washington, Feb 2007.
%$Id:$
%$DateTime:$
%$Author:$

% Initialize to empty model structure - single-centroid initialization
% works also for multi-centroid
modelParams = initCentroidModels;

% Compute model parameters
uniqueClassLabels = unique(classLabels);
nClasses = length(uniqueClassLabels);

% If only one class is observed, then the model is trivial and the
% classifier will always classify as this class. The classifier will need to
% know this by looking at the isTrivial field.
if nClasses == 1
  modelParams.isTrivial = true;
else
  modelParams.isTrivial = false;
end

% The following works for arbitrary number of classes
centroidIdx = [];
classCounts = zeros(nClasses, 1);
for iClass = 1:nClasses
  thisClassLabel = uniqueClassLabels(iClass);
  thisClassIdx = classLabels == thisClassLabel;
  thisCentroidIdx = find(thisClassIdx(initCentroidIdx));
  thisClassCentroidIdx = findMultipleCentroids( ...
    simMat(thisClassIdx, thisClassIdx), thisCentroidIdx, ...
    centroidType);
  
  % Convert: index into class submatrix --> index into entire simMat. 
  tmpIdx = find(thisClassIdx);
  tmpIdx = tmpIdx(:).'; %make sure it's a row vector
  centroidIdx = [centroidIdx tmpIdx(thisClassCentroidIdx)];
  
  % Count number of samples in each class
  classCounts(iClass) = sum(thisClassIdx);
end %for iClasses

% Sort the centroid indices - not needed, but very useful for mixture SDA 
centroidIdx = sort(centroidIdx);

% Put parameters in result structure
modelParams.centroidIdx = centroidIdx;
modelParams.centroidClassLabels = classLabels(centroidIdx);
modelParams.classLabels = uniqueClassLabels;
modelParams.centroidType = centroidType;
modelParams.classCounts = classCounts; 

% Note the Laplace correction for priors: added Jan 2009, LGC.
modelParams.classPriors = (classCounts + 1) / (sum(classCounts) + nClasses);

% Before Jan 2009
%modelParams.classPriors = classCounts / sum(classCounts);

return