function modelParams = computeNaiveBayesModels(dataMat, classLabels)
%computeNaiveBayesModels - estimate naive Bayes probabilities.
% Computes the naive Bayes probability mass functions from data, using
% histograms as approximations.
%
% USAGE 
% modelParams = computeNaiveBayesModels(dataMat, classLabels);
% dataMat: N-by-M matrix, each row is a sample and each column a feature.
% classLabels: N-by-1 vector of class labels for the rows of dataMat.
% modelParams: a structure containing the model parameters and ancillary
%     information. The fields are:
%     * isTrivial: boolean flag; true if only one class is represented in
%     the training data.
%     * classPriors: C-by-1 vector of class prior probabilities, where C is
%     the number of classes represented in classLabels.
%     * classCounts: C-by-1 vector of class frequency counts for the training
%     set.
%     * classLabels: C-by-1 vector of unique class labels.
%     * probModels: C-by-M cell array of probability models for each of the
%     M features in each of the C classes. Each element of probModels is a
%     vector of probabilities: the i-th entry is the probability of
%     occurrence of the i-th value of the m-th feature in the c-th class.
%     * featureVals: C-by-M cell array of possible feature values
%     corresponding to the probabilities in probModels.
%
% This function does not apply Laplace correction to the probability
% estimates, but its companion function classifyNaiveBayes() *does* apply
% it to both the class-conditional feature probablities and to the class
% priors, unless the model is trivial.
%
% See also initNaivebayes, classifyNaiveBayes.

% Written by Luca Cazzanti, engineer
% Applied Physics Laboratory, University of Washington, Dec 2006.
%$Id:$
%$DateTime:$
%$Author:$

% Initialize model parameter structure to a trivial empty fields
modelParams = initNaiveBayesModels;

%% Only one class --> then return a trivial probability model
[nData, nFeatures] = size(dataMat); %#ok
uniqueClassLabels = unique(classLabels);
nClasses = length(uniqueClassLabels);
if nClasses == 1
  modelParams.classPriors = 1;
  modelParams.classCounts = nData;
  modelParams.classLabels = uniqueClassLabels;
  % The other fields of modelParams are already initialized to [] by
  % initNaiveBayes() above.
  
  return
end

%% Non-trivial models: estimate pmfs
classCounts = zeros(1, nClasses);
featureVals = cell(nClasses, nFeatures);
probModels = cell(nClasses, nFeatures);

for iClass = 1:nClasses
  classIdx = find(classLabels == uniqueClassLabels(iClass));
  thisClassData = dataMat(classIdx,:); %#ok
  classSize = length(classIdx);
  classCounts(iClass) = classSize;
  for iFeature = 1:nFeatures
    uniqueFeatureValues = unique(thisClassData(:,iFeature));
    if length(uniqueFeatureValues) == 1 % feature takes on only one value
      featureHist = classSize;
    else
      featureHist = hist(thisClassData(:,iFeature), uniqueFeatureValues);
    end %if
    featureVals{iClass, iFeature} = uniqueFeatureValues;
    probModels{iClass, iFeature} = featureHist / classSize;
  end % for iFeature
end % for iClass

% Put parameters in model structure
modelParams.isTrivial = false;
modelParams.classCounts = classCounts;
modelParams.classPriors = classCounts / sum(classCounts);
modelParams.classLabels = uniqueClassLabels;
modelParams.probModels = probModels;
modelParams.featureVals = featureVals;


return