function [mostPopIdx, popularity] = findCentroidMostPopular(inMat, varargin)
%findCentroidMostPopular - find centroid based on popularity.
% Finds the centroid based on the popularity of the elements of a matrix of
% pairwise (dis)similarities. The popularity of an element is the number of
% other elements which rank it as most similar. Ties in popularity are
% broken by selecting the most popular element with the highest sum of
% similarities to the elements to which it is the most similar.
%
% USAGE
% [mostPopIdx, popularity] = findCentroidMostPopular(inMat, inMode)
% inMat - square matrix of pairwise (dis)similarities.
% inMode - string specifying if inMat is similarities or dissimilarities;
%     use 'similarity' (default) or 'dissimilarity' respectively.
% mostPopIdx - row index into inMat corresponding to the centroid.
% popularity - vector of popularities corresponding to each row of inMat.

% Written by Luca Cazzanti, engineer
% Applied Physics Laboratory - University of Washington, Nov. 2006.
%$Id:$
%$DateTime:$
%$Author:$

%% Check inputs
% ...matrix must be square...
[nRows, nCols]= size(inMat);
if nRows ~= nCols
  error('Input matrix must be square');
end

%% Order data by nearest neighbor
[sortIdx, sortMat] = findNearNeighbors(inMat, nCols, varargin);

%% Look for centroid based on popularity
% Assumption:  a point's self-(dis)similarity is always (less) greater than
% its similarities to other points, so its own index is always in the first
% column of the index matrix. The index of its nearest neighbor other than
% itself is therefore in the second column. 
nearNeighborIdx = sortIdx(:,2);

% ... compute popularity ...
histX = 1:nRows;
popularity = hist(nearNeighborIdx, histX);
[mostVotes, mostPopIdx] = max(popularity);

% ... almost done, one more detail: if there is more than one winner, break
% ties using maximum sum similarity ... 
winnerIdx = find(popularity == mostVotes);
nWinners = length(winnerIdx);
selfSim = diag(inMat);
if nWinners > 1
  winnerSim = zeros(nWinners,1);
  for idx = 1:nWinners
    elemsToSum = find( nearNeighborIdx == winnerIdx(idx) );
    winnerSim(idx) = sum(sortMat(elemsToSum,2)); %#ok
    % add self-similarity
    winnerSim(idx) = winnerSim(idx) + selfSim(winnerIdx(idx));
  end % for idx
  [bestSim, bestSimIdx] = sort(winnerSim, sortMode); %#ok
  mostPopIdx = winnerIdx(bestSimIdx(1)); % see NOTE below
end % if nWinners > 1

% NOTE: it could happen that a tie happens also with MaxSumSim; in this
% case the first element is picked.

return